@@ -36,6 +36,7 @@
 
 #include <time.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <string.h>
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
@@ -177,7 +178,7 @@ encodings[] =
 	{ N_("R_omanian (ISO-8859-16)"),            "ISO-8859-16",           LG_OTHER },
 	{ N_("South _European (ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER },
 	{ N_("Thai (TIS-_620)"),                    "TIS-620",               LG_OTHER },
-#if MOZILLA_SNAPSHOT >= 9
+#if MOZILLA_SNAPSHOT >= 10
 	{ N_("Thai (IS_O-8859-11)"),                "iso-8859-11",           LG_OTHER },
 	{ N_("_Thai (Windows-874)"),                "windows-874",           LG_OTHER },
 #endif	
@@ -328,9 +329,9 @@ mozilla_set_default_prefs (MozillaEmbedSingle *mes)
 	pref->SetBoolPref ("mozilla.widget.raise-on-setfocus", PR_FALSE);
 
 	/* set default search engine */
-	pref->SetCharPref ("keyword.URL", "http://www.google.com/search?q=");
+	pref->SetCharPref ("keyword.URL", "http://www.google.com/search?ie=UTF-8&oe=UTF-8&q=");
 	pref->SetBoolPref ("keyword.enabled", PR_TRUE);
-	pref->SetBoolPref ("security.checkloaduri", PR_FALSE);
+	pref->SetBoolPref ("security.checkloaduri", PR_TRUE);
 
 	/* dont allow xpi installs from epiphany, there are crashes */
 	pref->SetBoolPref ("xpinstall.enabled", PR_FALSE);
@@ -1053,20 +1054,22 @@ impl_show_file_picker (EphyEmbedSingle *shell,
                        FileFormat *file_formats, 
 		       int *ret_file_format)
 {
-	gchar *expanded_directory;
+	gchar *expanded_directory = NULL;
 	gresult result;
 
         GFilePicker *filePicker = new GFilePicker (file_formats);
 
-	/* FIXME sane path: expand tilde ... */
-        expanded_directory = g_strdup (directory);
+	if (directory != NULL)
+	{
+		expanded_directory = gnome_vfs_expand_initial_tilde (directory);
+	}
 
         /* make sure the directory exists, and use the home directory
          * otherwise */
         if (!expanded_directory ||
             !g_file_test (expanded_directory, G_FILE_TEST_IS_DIR))
         {
-                if (expanded_directory) g_free (expanded_directory);
+                g_free (expanded_directory);
                 expanded_directory = g_strdup (g_get_home_dir());
         }
 
