@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-shell.c,v 1.54.2.1 2003/09/24 20:39:52 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,7 +36,12 @@
 #include "ephy-bookmarks-editor.h"
 #include "ephy-history-window.h"
 #include "ephy-debug.h"
+#include "ephy-plugin.h"
 #include "toolbar.h"
+#include "session.h"
+#include "downloader-view.h"
+#include "ephy-toolbars-model.h"
+#include "ephy-autocompletion.h"
 
 #include <string.h>
 #include <libgnomeui/gnome-client.h>
@@ -43,6 +50,7 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmessagedialog.h>
+#include <dirent.h>
 
 #ifdef ENABLE_NAUTILUS_VIEW
 
@@ -61,13 +69,8 @@ struct EphyShellPrivate
 	EphyToolbarsModel *toolbars_model;
 	GtkWidget *bme;
 	GtkWidget *history_window;
-};
-
-enum
-{
-        STARTPAGE_HOME,
-        STARTPAGE_LAST,
-        STARTPAGE_BLANK,
+	GList *plugins;
+	gboolean server_mode;
 };
 
 static void
@@ -133,84 +136,6 @@ ephy_shell_class_init (EphyShellClass *klass)
         object_class->finalize = ephy_shell_finalize;
 }
 
-static void
-ephy_shell_command_cb (EphyEmbedShell *shell,
-		       char *command,
-		       char *param,
-		       gpointer data)
-{
-	EphyBookmarks *bookmarks;
-	GtkWidget *dialog;
-	const gchar *message = NULL;
-	GtkMessageType message_type = GTK_MESSAGE_INFO;
-	EphyWindow *window;
-
-	bookmarks = ephy_shell_get_bookmarks (EPHY_SHELL (shell));
-
-	if (strcmp (command, "import-mozilla-bookmarks") == 0)
-	{
-		if (ephy_bookmarks_import_mozilla (bookmarks, param))
-		{
-			message_type =  GTK_MESSAGE_INFO;
-			message = _("Mozilla bookmarks imported successfully.");
-		}
-		else
-		{
-			message_type = GTK_MESSAGE_ERROR;
-			message = _("Importing Mozilla bookmarks failed.");
-		}
-	}
-	else if (strcmp (command, "import-galeon-bookmarks") == 0)
-	{
-		if (ephy_bookmarks_import_xbel (bookmarks, param,
-						_("Galeon")))
-		{
-			message_type = GTK_MESSAGE_INFO;
-			message = _("Galeon bookmarks imported successfully.");
-		}
-		else
-		{
-			message_type = GTK_MESSAGE_ERROR;
-			message = _("Importing Galeon bookmarks failed.");
-		}
-	}
-	else if (strcmp (command, "import-konqueror-bookmarks") == 0)
-	{
-		if (ephy_bookmarks_import_xbel (bookmarks, param,
-						_("Konqueror")))
-		{
-			message_type = GTK_MESSAGE_INFO;
-			message = _("Konqueror bookmarks imported successfully.");
-		}
-		else
-		{
-			message_type = GTK_MESSAGE_ERROR;
-			message = _("Importing Konqueror bookmarks failed.");
-		}
-	}
-	else if (strcmp (command, "configure-network") == 0)
-	{
-		ephy_file_launch_application ("gnome-network-preferences",
-					      NULL,
-					      FALSE);
-	}
-
-	if (message != NULL)
-	{
-		window = ephy_shell_get_active_window (EPHY_SHELL (shell));
-		dialog = gtk_message_dialog_new (GTK_WINDOW (window),
-				 		 GTK_DIALOG_DESTROY_WITH_PARENT,
-				 		 message_type,
-				 		 GTK_BUTTONS_OK,
-				 		 message);
-		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-		g_signal_connect_swapped (G_OBJECT (dialog), "response",
-                           		  G_CALLBACK (gtk_widget_destroy),
-                           		  G_OBJECT (dialog));
-		gtk_widget_show (dialog);
-	}
-}
-
 static void
 ephy_shell_new_window_cb (EphyEmbedShell *shell,
 			  EphyEmbed **new_embed,
@@ -219,33 +144,55 @@ ephy_shell_new_window_cb (EphyEmbedShell *shell,
 {
 	EphyTab *new_tab;
         EphyWindow *window;
-	gboolean open_in_tab;
 
 	g_assert (new_embed != NULL);
 
-	open_in_tab = (chromemask & EMBED_CHROME_OPENASCHROME) ?
-		      FALSE :
-		      eel_gconf_get_boolean (CONF_TABS_TABBED_POPUPS);
-
-	if (open_in_tab)
-	{
-		window = ephy_shell_get_active_window (ephy_shell);
-	}
-	else
-	{
-		window = ephy_window_new ();
-		ephy_window_set_chrome (window, chromemask);
-	}
+	window = ephy_window_new ();
+	ephy_window_set_chrome (window, chromemask);
 
 	new_tab = ephy_tab_new ();
-	ephy_window_add_tab (window, new_tab, TRUE, FALSE);
+	ephy_window_add_tab (window, new_tab, EPHY_NOTEBOOK_INSERT_GROUPED, FALSE);
 	*new_embed = ephy_tab_get_embed (new_tab);
 }
 
+static void
+ephy_shell_load_plugins (EphyShell *es)
+{
+	DIR *d;
+	struct dirent *e;
+
+	d = opendir (PLUGINS_DIR);
+
+	if (d == NULL)
+	{
+		return;
+	}
+
+	while ((e = readdir (d)) != NULL)
+	{
+		char *plugin;
+
+		plugin = g_strconcat (PLUGINS_DIR"/", e->d_name, NULL);
+
+		if (g_str_has_suffix (plugin, ".so"))
+		{
+			EphyPlugin *obj;
+
+			obj = ephy_plugin_new (plugin);
+			es->priv->plugins = g_list_append
+				(es->priv->plugins, obj);
+		}
+
+		g_free (plugin);
+	}
+	closedir (d);
+}
+
 static void
 ephy_shell_init (EphyShell *gs)
 {
 	EphyEmbedSingle *single;
+	EphyShell **ptr = &ephy_shell;
 
         gs->priv = g_new0 (EphyShellPrivate, 1);
 	gs->priv->session = NULL;
@@ -253,44 +200,48 @@ ephy_shell_init (EphyShell *gs)
 	gs->priv->bme = NULL;
 	gs->priv->history_window = NULL;
 	gs->priv->toolbars_model = NULL;
+	gs->priv->plugins = NULL;
+	gs->priv->server_mode = FALSE;
 
 	ephy_shell = gs;
 	g_object_add_weak_pointer (G_OBJECT(ephy_shell),
-				   (gpointer *)&ephy_shell);
+				   (gpointer *)ptr);
 
 	ephy_debug_init ();
 	ephy_thread_helpers_init ();
-	ephy_node_system_init (EPHY_NODE_RESERVED_IDS);
 	ephy_file_helpers_init ();
 	ephy_stock_icons_init ();
 	ephy_ensure_dir_exists (ephy_dot_dir ());
 
 	/* This ensures mozilla is fired up */
 	single = ephy_embed_shell_get_embed_single (EPHY_EMBED_SHELL (gs));
+	if (single != NULL)
+	{
+		g_signal_connect (G_OBJECT (single),
+				  "new_window_orphan",
+				  G_CALLBACK(ephy_shell_new_window_cb),
+				  NULL);
 
-	g_signal_connect (G_OBJECT (single),
-			  "new_window_orphan",
-			  G_CALLBACK(ephy_shell_new_window_cb),
-			  NULL);
+		ephy_init_services (gs);
+	}
+	else
+	{
+		GtkWidget *dialog;
 
-	g_signal_connect (G_OBJECT (gs),
-			  "command",
-			  G_CALLBACK(ephy_shell_command_cb),
-			  NULL);
+		dialog = gtk_message_dialog_new
+			(NULL,
+                         GTK_DIALOG_MODAL,
+                         GTK_MESSAGE_ERROR,
+                         GTK_BUTTONS_CLOSE,
+                         _("Epiphany can't be used now. "
+                         "Mozilla initialization failed. Check your "
+                         "MOZILLA_FIVE_HOME environmental variable."));
+		gtk_dialog_run (GTK_DIALOG (dialog));
 
-	ephy_init_services (gs);
-}
+		exit (0);
+	}
 
-static void
-save_toolbars (EggToolbarsModel *model)
-{
-	char *xml_file;
-
-	xml_file = g_build_filename (ephy_dot_dir (),
-                                     "ephy-toolbar.xml",
-                                     NULL);
-	egg_toolbars_model_save (model, xml_file);
-	g_free (xml_file);
+	ephy_shell_load_plugins (gs);
 }
 
 static void
@@ -307,10 +258,12 @@ ephy_shell_finalize (GObject *object)
 
 	g_assert (ephy_shell == NULL);
 
+	g_list_foreach (gs->priv->plugins, (GFunc)g_type_module_unuse, NULL);
+	g_list_free (gs->priv->plugins);
+
 	LOG ("Unref toolbars model")
 	if (gs->priv->toolbars_model)
 	{
-		save_toolbars (EGG_TOOLBARS_MODEL (gs->priv->toolbars_model));
 		g_object_unref (G_OBJECT (gs->priv->toolbars_model));
 	}
 
@@ -354,7 +307,6 @@ ephy_shell_finalize (GObject *object)
 
 	ephy_state_save ();
 	ephy_file_helpers_shutdown ();
-	ephy_node_system_shutdown ();
 
 	LOG ("Ephy shell finalized")
 
@@ -375,7 +327,6 @@ ephy_init_services (EphyShell *gs)
 	/* preload the prefs */
 	/* it also enables notifiers support */
 	eel_gconf_monitor_add ("/apps/epiphany");
-	eel_gconf_monitor_add ("/apps/nautilus/preferences");
 	eel_gconf_monitor_add ("/system/proxy");
 
 #ifdef ENABLE_NAUTILUS_VIEW
@@ -386,58 +337,23 @@ ephy_init_services (EphyShell *gs)
 
 }
 
-static char *
-build_homepage_url (EphyShell *gs,
-		    EphyEmbed *previous_embed)
+static void
+load_homepage (EphyEmbed *embed)
 {
-        const gchar *last_page_url;
-        gchar *home_page_url;
-        gint page_type;
-        EphyHistory *gh;
-	char *result = NULL;
+	char *home;
 
-        if (previous_embed == NULL)
-        {
-                page_type = STARTPAGE_HOME;
-        }
-        else
-        {
-                page_type = eel_gconf_get_integer (CONF_GENERAL_NEWPAGE_TYPE);
-        }
+	home = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
 
-        /* return the appropriate page */
-        if (page_type == STARTPAGE_HOME)
-        {
-                /* get location of home page */
-                home_page_url = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
-		result = home_page_url;
-        }
-	else if (page_type == STARTPAGE_LAST)
+	if (home == NULL || home[0] == '\0')
 	{
-		if (previous_embed != NULL)
-		{
-			ephy_embed_get_location (previous_embed,
-						 TRUE,
-						 &result);
-		}
+		g_free (home);
 
-		if (result == NULL)
-		{
-			/* get location of last page */
-			gh = ephy_embed_shell_get_global_history
-				(EPHY_EMBED_SHELL (gs));
-			last_page_url = ephy_history_get_last_page (gh);
-			result = g_strdup (last_page_url);
-		}
+		home = g_strdup ("about:blank");
 	}
 
-	if (result == NULL)
-	{
-		/* even in case of error, it's a good default */
-		result = g_strdup ("about:blank");
-	}
+	ephy_embed_load_url (embed, home);
 
-	return result;
+	g_free (home);
 }
 
 /**
@@ -457,7 +373,7 @@ ephy_shell_get_active_window (EphyShell *gs)
 	Session *session;
 	const GList *windows;
 
-	session = ephy_shell_get_session (gs);
+	session = SESSION (ephy_shell_get_session (gs));
 	windows = session_get_windows (session);
 
 	if (windows == NULL) return NULL;
@@ -488,19 +404,19 @@ ephy_shell_new_tab (EphyShell *shell,
 	EphyTab *tab;
 	EphyEmbed *embed;
 	gboolean in_new_window;
-	gboolean grouped = FALSE;
+	gboolean grouped;
 	gboolean jump_to;
 	EphyEmbed *previous_embed = NULL;
+	GtkWidget *nb;
+	gint position;
+	Toolbar *toolbar;
 
 	in_new_window = !eel_gconf_get_boolean (CONF_TABS_TABBED);
 
 	if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
 	if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
 
-	jump_to = eel_gconf_get_boolean (CONF_TABS_TABBED_AUTOJUMP);
-
-	if (flags & EPHY_NEW_TAB_JUMP) jump_to = TRUE;
-	if (flags & EPHY_NEW_TAB_DONT_JUMP_TO) jump_to = FALSE;
+	jump_to = (flags & EPHY_NEW_TAB_JUMP);
 
 	if (!in_new_window && parent_window != NULL)
 	{
@@ -511,69 +427,63 @@ ephy_shell_new_tab (EphyShell *shell,
 		window = ephy_window_new ();
 	}
 
+	toolbar = ephy_window_get_toolbar (window);
+
 	if (previous_tab)
 	{
 		previous_embed = ephy_tab_get_embed (previous_tab);
 	}
 
-	if (url != NULL || flags & EPHY_NEW_TAB_IS_A_COPY ||
-	    flags & EPHY_NEW_TAB_VIEW_SOURCE)
-	{
-		grouped = TRUE;
-	}
+	grouped = ((flags & EPHY_NEW_TAB_OPEN_PAGE ||
+	            flags & EPHY_NEW_TAB_APPEND_GROUPED ||
+		    flags & EPHY_NEW_TAB_CLONE_PAGE)) &&
+		  !(flags & EPHY_NEW_TAB_APPEND_LAST);
 
-	if (flags & EPHY_NEW_TAB_APPEND_GROUPED) grouped = TRUE;
-	if (flags & EPHY_NEW_TAB_APPEND_LAST) grouped = FALSE;
+	if ((flags & EPHY_NEW_TAB_APPEND_AFTER) && previous_embed != NULL)
+	{
+		nb = ephy_window_get_notebook (window);
+		position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
+						  GTK_WIDGET (previous_embed)) + 1;
+	}
+	else
+	{
+		position = grouped ? EPHY_NOTEBOOK_INSERT_GROUPED : EPHY_NOTEBOOK_INSERT_LAST;
+	}
 
 	tab = ephy_tab_new ();
 	embed = ephy_tab_get_embed (tab);
 	gtk_widget_show (GTK_WIDGET(embed));
 	ephy_window_add_tab (window, tab,
-			     grouped,
+			     position,
 			     jump_to);
+	gtk_widget_show (GTK_WIDGET(window));
 
-	if (flags & EPHY_NEW_TAB_RAISE_WINDOW)
+	if (flags & EPHY_NEW_TAB_HOME_PAGE ||
+	    flags & EPHY_NEW_TAB_NEW_PAGE)
 	{
-		gtk_window_present (GTK_WINDOW(window));
+		ephy_tab_set_location (tab, "", TAB_ADDRESS_EXPIRE_NEXT);
+		toolbar_activate_location (toolbar);
+		load_homepage (embed);
 	}
-	else
+	else if (flags & EPHY_NEW_TAB_OPEN_PAGE)
 	{
-		gtk_widget_show (GTK_WIDGET(window));
+		g_assert (url != NULL);
+		ephy_embed_load_url (embed, url);
 	}
-
-	if (flags & EPHY_NEW_TAB_HOMEPAGE)
+	else if (flags & EPHY_NEW_TAB_CLONE_PAGE)
 	{
-		Toolbar *toolbar;
-		char *homepage;
+		EmbedDisplayType display_type;
 
-		homepage = build_homepage_url (shell, previous_embed);
-		g_assert (homepage != NULL);
+                display_type = (flags & EPHY_NEW_TAB_SOURCE_MODE) ?
+                                DISPLAY_AS_SOURCE : DISPLAY_NORMAL;
 
-		toolbar = ephy_window_get_toolbar (window);
-		toolbar_edit_location (toolbar);
-
-		ephy_embed_load_url (embed, homepage);
-
-		g_free (homepage);
-	}
-	else if ((flags & EPHY_NEW_TAB_IS_A_COPY) ||
-		 (flags & EPHY_NEW_TAB_VIEW_SOURCE))
-	{
-		EmbedDisplayType display_type =
-			(flags & EPHY_NEW_TAB_VIEW_SOURCE) ?
-			 DISPLAY_AS_SOURCE : DISPLAY_NORMAL;
 		ephy_embed_load_url (embed, "about:blank");
 		ephy_embed_copy_page (embed, previous_embed, display_type);
 	}
-	else if (url)
-	{
-		ephy_embed_load_url (embed, url);
-	}
 
-	if (flags & EPHY_NEW_TAB_FULLSCREEN)
+	if (flags & EPHY_NEW_TAB_FULLSCREEN_MODE)
 	{
-		ephy_window_set_chrome (window, EMBED_CHROME_OPENASFULLSCREEN |
-				        EMBED_CHROME_DEFAULT);
+		gtk_window_fullscreen (GTK_WINDOW (window));
 	}
 
         return tab;
@@ -610,7 +520,7 @@ ephy_nautilus_view_new (BonoboGenericFactory *factory, const char *id,
  *
  * Return value: the current session.
  **/
-Session *
+GObject *
 ephy_shell_get_session (EphyShell *gs)
 {
 	if (!gs->priv->session)
@@ -621,10 +531,10 @@ ephy_shell_get_session (EphyShell *gs)
                          (gpointer *)&gs->priv->session);
 	}
 
-	return gs->priv->session;
+	return G_OBJECT (gs->priv->session);
 }
 
-EphyAutocompletion *
+GObject *
 ephy_shell_get_autocompletion (EphyShell *gs)
 {
 	EphyShellPrivate *p = gs->priv;
@@ -646,7 +556,7 @@ ephy_shell_get_autocompletion (EphyShell *gs)
 		ephy_autocompletion_add_source (p->autocompletion,
 						EPHY_AUTOCOMPLETION_SOURCE (bmk));
 	}
-	return p->autocompletion;
+	return G_OBJECT (p->autocompletion);
 }
 
 EphyBookmarks *
@@ -660,35 +570,22 @@ ephy_shell_get_bookmarks (EphyShell *gs)
 	return gs->priv->bookmarks;
 }
 
-EphyToolbarsModel *
+GObject *
 ephy_shell_get_toolbars_model (EphyShell *gs)
 {
 	if (gs->priv->toolbars_model == NULL)
 	{
-		char *xml_file;
-		EggToolbarsModel *model;
+		EphyBookmarks *bookmarks;
 
-		gs->priv->toolbars_model = ephy_toolbars_model_new ();
-		model = EGG_TOOLBARS_MODEL (gs->priv->toolbars_model);
+		bookmarks = ephy_shell_get_bookmarks (gs);
 
-		xml_file = g_build_filename (ephy_dot_dir (),
-		                             "ephy-toolbar.xml",
-		                             NULL);
-		if (g_file_test (xml_file, G_FILE_TEST_EXISTS))
-		{
-			egg_toolbars_model_load (model, xml_file);
-		}
-		else
-		{
-			const char *default_xml;
+		gs->priv->toolbars_model = ephy_toolbars_model_new (bookmarks);
 
-			default_xml = ephy_file ("epiphany-toolbar.xml");
-			egg_toolbars_model_load (model, default_xml);
-		}
-		g_free (xml_file);
+		g_object_set (bookmarks, "toolbars_model",
+			      gs->priv->toolbars_model, NULL);
 	}
 
-	return gs->priv->toolbars_model;
+	return G_OBJECT (gs->priv->toolbars_model);
 }
 
 static void
@@ -766,3 +663,27 @@ ephy_shell_show_history_window (EphyShell *gs,
 	gtk_window_present (GTK_WINDOW (gs->priv->history_window));
 }
 
+gboolean
+ephy_shell_get_server_mode (EphyShell *es)
+{
+	g_return_val_if_fail (IS_EPHY_SHELL (es), FALSE);
+
+	return es->priv->server_mode;
+}
+
+void
+ephy_shell_set_server_mode (EphyShell *es, gboolean as_server)
+{
+	g_return_if_fail (IS_EPHY_SHELL (es));
+
+	if (es->priv->server_mode == FALSE && as_server == TRUE)
+	{
+		g_object_ref (G_OBJECT (es));
+	}
+	else if (es->priv->server_mode == TRUE && as_server == FALSE)
+	{
+		g_object_unref (G_OBJECT (es));
+	}
+
+	es->priv->server_mode = as_server;
+}
